@@ -10,6 +10,7 @@ import subprocess
 import site
 import webbrowser
 from datetime import datetime
+from pathlib import Path
 from json import load as json_load
 from json import dumps as json_dumps
 from dsframework.documentation.scripts import doxygen_handler
@@ -82,6 +83,10 @@ def cli():
     ====================
     dsf-cli g predictable my-new-predictable
 
+    Generate stage
+    ====================
+    dsf-cli g stage my-new-stage
+
     Create files
     ===================
     dsf-cli create-files [tester / deploy / cloud_eval / documentation / batch / trainer]
@@ -162,7 +167,7 @@ def cli():
 def generate(gen_type, project_name):
     """! Build a generation function from the incoming gen_type.
         Args:
-            gen_type: A string containing one of the following: [project / forcer / predictable / documentation]
+            gen_type: A string containing one of the following: [project / forcer / predictable / documentation / stage]
             project_name: A string declaring our project name
     """
     try:
@@ -455,16 +460,42 @@ def generate_project(project_name):
     create_project(project_name)
 
 
-def generate_forcer(fileName):
+def generate_forcer(file_name):
     """! Generate new forcer - can help us, if we need forcer on an existing project """
-    fileName = clean_name(fileName)
-    create_exist_pipeline_file('forcer', fileName)
+    file_name = clean_name(file_name)
+    create_exist_pipeline_file('forcer', file_name)
 
 
-def generate_predictable(fileName):
-    """! Generate new predictable - can help us, if we need predictable on an existing project """
-    fileName = clean_name(fileName)
-    create_exist_pipeline_file('predictable', fileName)
+def generate_predictable(file_name):
+    """! Generate new predictable - can help us, if we need predictable on an existing project
+    """
+    file_name = clean_name(file_name)
+    create_exist_pipeline_file('predictable', file_name)
+
+
+def stage_name_already_in_use(stage_name):
+    """! Checks if a stage with the given name already exists
+    """
+    if os.path.exists('batch_files/stages/' + stage_name):
+        click.echo('Stage \'' + stage_name + '\' already exists. Please use a different name.')
+        return True
+    return False
+
+
+def generate_stage(stage_name):
+    """! Generating new stage (with the relevant project_name)\n
+    How to generate stage :
+    > dsf-cli g stage my-new-stage
+    @verbatim
+    Args:
+            stage_name: Stage name to be used when creating files / folders / classes.
+    @endverbatim
+    """
+    stage_name = clean_name(stage_name)
+    if stage_name_already_in_use(stage_name):
+        return
+    create_stage_files_from_template(stage_name)
+    click.echo('Stage created: ' + stage_name)
 
 
 def generate_documentation(simple="full"):
@@ -741,34 +772,34 @@ def create_project(project_name):
     run_git_init()
 
 
-def create_main_folders(targetDir, baseDir):
+def create_main_folders(target_dir, base_dir):
     """! Create folder if not already exists
     @verbatim
     Args:
-        targetDir: the new folder to create
-        baseDir  : where we want to create the new folder.
+        target_dir: the new folder to create
+        base_dir  : where we want to create the new folder.
     @endverbatim
     """
 
     global directories
-    directories[targetDir] = directories[baseDir] + '/' + targetDir
-    if not os.path.exists(directories[targetDir]):
-        os.mkdir(directories[targetDir])
+    directories[target_dir] = directories[base_dir] + '/' + target_dir
+    if not os.path.exists(directories[target_dir]):
+        os.mkdir(directories[target_dir])
 
 
-def create_pipeline_file(project_name, folder, pipelineType, createInitFile=True):
+def create_pipeline_file(project_name, folder, pipeline_type, create_init_file=True):
     """! Takes the template file and replace it with new one with project_name
     @verbatim
     Args:
         project_name : The project name
         folder       : The folder we want to create the new file.
-        pipelineType : File type in the pipeline.
-        createInitFile(=True): Create init file - if doesnt exist.
+        pipeline_type : File type in the pipeline.
+        create_init_file(=True): Create init file - if doesnt exist.
     @endverbatim
     """
 
-    data = read_template_file(pipelineType)
-    replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile)
+    data = read_template_file(pipeline_type)
+    replace_in_template_and_create_file(project_name, folder, pipeline_type, data, create_init_file)
 
 
 def create_schema_file(project_name, folder, pipelineType, createInitFile=True):
@@ -875,6 +906,20 @@ def create_exist_pipeline_file(type, fileName):
     pass
 
 
+def create_stage_files_from_template(stage_name):
+    """! Create all needed folders and files for the new stage creation
+    @verbatim
+    Args:
+        stage_name: Stage name
+    @endverbatim
+    """
+
+    data = read_template_file('batch/stage')
+    stage_directory_name = 'batch_files/stages/' + stage_name
+    os.mkdir(stage_directory_name)
+    replace_in_stage_template_and_create_file(stage_name, stage_directory_name, data)
+
+
 def create_tester_env():
     """! This function will create a testing environment, if one does not already exist in the project.
     It will create the testing, evaluation, and relevant schema files.
@@ -946,45 +991,59 @@ def write_to_file(file_path, data):
         file.close()
 
 
-def replace_in_template_and_create_file(project_name, folder, pipelineType, data, createInitFile):
+def replace_in_template_and_create_file(project_name, folder, file_name, data, with_init_file):
     """! Get & Replace the template file and create new one with project_name
     @verbatim
     Args:
         project_name : The project name
         folder       : The folder we want to create the new file.
-        pipelineType : Which File.
+        file_name : Which File.
         data         : The template file.
-        createInitFile(=True): Create init file - if doesnt exist.
+        with_init_file(=True): Create init file if doesn't exist.
     @endverbatim
     """
 
-    pipeline_type_no_underscore = to_capitalize_no_underscore(pipelineType)
+    pipeline_type_no_underscore = to_capitalize_no_underscore(file_name)
     project_name_no_underscore = to_capitalize_no_underscore(project_name)
-    className = project_name_no_underscore + pipeline_type_no_underscore
-    classNameForBaseObject = project_name_no_underscore + pipeline_type_no_underscore
-    currentDir = folder.replace('/', '.')
-    currentDirNoMainDir = folder.replace(directories['main'] + '/', '').replace('/', '.')
-    currentBaseDir = directories['main'].replace('/', '.')
-    currentPipelineDir = directories['pipeline'].replace('/', '.')
+    class_name = project_name_no_underscore + pipeline_type_no_underscore
+    class_name_for_base_object = project_name_no_underscore + pipeline_type_no_underscore
+    current_dir = folder.replace('/', '.')
+    current_base_dir = directories['main'].replace('/', '.')
+    current_pipeline_dir = directories['pipeline'].replace('/', '.')
 
-    data = data.replace('generatedClassName', classNameForBaseObject)
-    data = data.replace('generatedClass', className)
+    data = data.replace('generatedClassName', class_name_for_base_object)
+    data = data.replace('generatedClass', class_name)
     data = data.replace('original_project_name', project_name)
     data = data.replace('generatedProjectName', project_name_no_underscore)
-    data = data.replace('generatedDirectory', currentDir)
-    data = data.replace('generatedBaseDir', currentBaseDir)
-    data = data.replace('generatedPipelineDir', currentPipelineDir)
+    data = data.replace('generatedDirectory', current_dir)
+    data = data.replace('generatedBaseDir', current_base_dir)
+    data = data.replace('generatedPipelineDir', current_pipeline_dir)
 
-    new_file = folder + "/" + pipelineType + ".py"
+    new_file = folder + "/" + file_name + ".py"
     create_file(new_file, data)
 
-    if createInitFile:
+    if with_init_file:
         new_init_file = folder + "/__init__.py"
-        new_init_export = "from " + '.' + pipelineType + " import " + className
+        new_init_export = "from " + '.' + file_name + " import " + class_name
         create_init_file(new_init_file, new_init_export)
 
 
-def create_empty_init_file(folder):
+def replace_in_stage_template_and_create_file(stage_name, folder, data):
+    """! Get & Replace the stage template file and create a new one with stage_name
+    @verbatim
+    Args:
+        stage_name : The stage name
+        folder     : The destination folder for the new files
+        data       : The template file
+    @endverbatim
+    """
+
+    data = data.replace('generatedStageName', to_capitalize_no_underscore(stage_name))
+    create_file(folder + "/main.py", data)
+    create_empty_init_file(folder)
+
+
+def create_empty_init_file(folder: str):
     """Creates an empty __init__.py file
     Args:
         folder       : The folder we want to create the new file in.
@@ -1014,7 +1073,7 @@ def create_file(new_file_path, data):
 
 
 def create_init_file(init_path, init_export):
-    """! if init file doesnt exist - create it and add the export line
+    """! if init file doesn't exist - create it and add the export line
          if init file exist - check if the init_export line appears in the init file : if not add it.
 
     @verbatim
@@ -1321,13 +1380,7 @@ def copy_cloud_eval_files(main_dir):
 
 
 def copy_batch_files(main_dir):
-    """! Generate batch files needed for batch projects. Creates 3 stages:
-    1. Get Data
-    2. Run Pipeline
-    3. Set Data
-
-    Along with a matching workflow file, and a configuration file
-
+    """! Generate batch files needed for batch projects.
         Args:
             main_dir: A string describing the main working directory, on existing projects, it's empty.
     """
@@ -1346,6 +1399,7 @@ def copy_batch_files(main_dir):
         dir_path = dir_path.replace(source_dir, '')
         file_list = [os.path.join(dir_path, file) for file in filenames]
         for file_path in file_list:
+            file_suffix = Path(file_path).suffix
             with open(os.path.join(source_dir, file_path), 'r', encoding="utf8") as file:
                 try:
                     data = file.read()
@@ -1368,8 +1422,10 @@ def copy_batch_files(main_dir):
                         f = open(target_dir + file_path, "w")
                         f.write(data)
                         f.close()
+                        if file_suffix == '.sh':
+                            os.chmod(target_dir + file_path, 0o744)
                 except Exception as ex:
-                    if file_path.rsplit('.', 1)[-1] == 'jar':
+                    if file_suffix == '.jar':
                         shutil.copy(os.path.join(source_dir, file_path), f'{dir_to_create}')
                     else:
                         print(f'Error when trying to open: {file_path}: {ex}')
