@@ -46,30 +46,20 @@ class Workflow(ZIDS_Workflow):
 
         return stage_job
 
-    def create_stages(self) -> List[OrderedJob]:
+    def create_jobs(self) -> List[OrderedJob]:
+        """! Generates a job for each of the workflow stages
+            Returns:
+                A List of OrderedJob, to be inserted into the workflow template
+        """
+        example_job = self.create_job("example_stage")
+
+        return [example_job]
+
+    def create_trainer_jobs(self) -> List[OrderedJob]:
         """! Create the stages for the workflow stages
             Returns:
                 A List of Ordered Jobs, to be inserted into the workflow template
         """
-        # stage get_data
-        stage_get_data_job = self.create_job("get_data")
-
-        # stage run_pipeline
-        prerequisite_list = [stage_get_data_job.step_id]
-        stage_run_pipeline_job = self.create_job("run_pipeline", prerequisite_list)
-
-        # stage set_data
-        prerequisite_list = [stage_run_pipeline_job.step_id]
-        stage_set_data_job = self.create_job("set_data", prerequisite_list)
-
-        return [stage_get_data_job, stage_run_pipeline_job, stage_set_data_job]
-
-    def create_trainer_stage(self) -> List[OrderedJob]:
-        """! Create the stages for the workflow stages
-            Returns:
-                A List of Ordered Jobs, to be inserted into the workflow template
-        """
-        # stage get_data
         self.whl_file_name = self.project_name + '_trainer.zip'
         stage_run_trainer_job = self.create_job("run_trainer")
 
@@ -127,9 +117,9 @@ class Workflow(ZIDS_Workflow):
         workflow_template = self.create_template_structure()
 
         if not self.trainer:
-            workflow_template.jobs = self.create_stages()
+            workflow_template.jobs = self.create_jobs()
         else:
-            workflow_template.jobs = self.create_trainer_stage()
+            workflow_template.jobs = self.create_trainer_jobs()
 
         self.finalize_jobs(workflow_template)
         return workflow_template
